####################################################################################################
# This R-script performs individual baed simulations to estimate the probability of a new          #
# mutation in a changing environment with variation among mutant copies within a generation.       #
#                                                                                                  #
# Fitness of the mutant changes from 1 + s0 to 1 + s. Fitness of a mutant in generation g is drawn #
# from a normal distribution with mean                                                             #
#  1 + s * (1-exp((-kappa) * g)) + s0 * exp((-kappa) * g)                                          #
# and standard deviation sd.                                                                       #  
#                                                                                                  #
# The script plots the probability of establishment as a function of the parameter s0              #
# The plots show the probability of establishment calcualted from the simulations, the             #
# correpsonding 99% confidence intervals, and the analytical prediciton as given in Peischl        #
# and Kirkpatrick (2012).                                                                          #
#                                                                                                  #
####################################################################################################


#population size
pop_size<-50000

#growth rate 
r<-2

#number of fixation events
number_fix<-0

#selection paramters
s<-0.01
s0 <- 0
rho<-exp(-0.01)    # rho = exp(- kappa)
sd<-0.1

#number of parameter values for s0
steps <- 5

#number of replicates
reps<-10000

# vector with values for s0
s0vec<- c(0,0.005,0.01,0.015,0.02)

# vector for fixation probability
fix<- c(0,0,0,0,0)


for (ii in c(1:steps))
{
  s0<-s0vec[ii]
  
  for (kk in c(1:reps))
  {
    
    jj<--1                   # jj counts generations
    
    muts <- 1
    resi <- pop_size - muts
    tot_ind<-resi+muts
    
    while(muts>0)      # until extinction
    { 
      jj<-jj+1         # increment generations  

      mean<-1+s0*(rho^jj)+s*(1-rho^jj)      # mean fitness of a mutant, relative to resident
  
      lambda <- r/( 1 + (tot_ind*(r-1)/pop_size))     # expected numer of offspring of a resident
     
      offspring <- rpois(resi,lambda)         # total number of resident offspring  
      
      resi<- sum(offspring)
      
      
      muts_next<-0      # number of mutants in the next generation
      
      for (i in c(1:muts))   # run through all mutants
      {
    
        #expected number of offspring of mutant i
        lambda_mut <-lambda*rnorm(1,mean,sd)
      
        #realized number of offspring of mutant i
        offspring <- rpois(1,max(0,lambda_mut))
        
        if(offspring>0 )
        {
          muts_next<-muts_next+offspring
        }
     
      }
      
      muts<- muts_next
      
      tot_ind<-resi+muts
    
      if(muts>500)   # stop if number of mutants is > 500, count this as fixation
      {
        muts<-0
        fix[ii] <- fix[ii] +1
      }
    }
  }
  }


qv<-fix/reps
w<-2.57* sqrt(qv *(1-qv)/reps) 

# plot confidence intervals
plotCI(s0vec,qv,uiw=w,col="black",xlab=expression(s[0]),ylab=expression(p),lwd=2,pch=20,gap=0)

#plot analytical prediction
lines(s0vec, 2*s*( 1 +(s0vec-s)/(1- (1-s)*rho) ),lwd=2,lty=1)